package Bundle::PlRPC;

$VERSION = '0.03';

1;

__END__

=head1 NAME

Bundle::PlRPC - A bundle to install PlRPC-Server, Client and prerequisites.

=head1 SYNOPSIS

C<perl -MCPAN -e 'install Bundle::PlRPC'>

=head1 CONTENTS

Storable

Net::Daemon

RPC::PlServer

=head1 DESCRIPTION

This bundle includes all that's needed to run PlRPC-Server and Client.

